@@ -21,7 +21,6 @@
 #include <sys/stat.h>
 #include <limits.h>
 #include <ctype.h>
-#include <time.h>
 
 /* locally defined functions */
 static INT s_errno PROTO((void));
@@ -877,64 +876,70 @@ static char *s_getwd() {
 
 static ptr s_set_code_byte(p, n, x) ptr p, n, x; {
     I8 *a;
+    ptr tc = get_thread_context();
 
-    S_thread_start_code_write();
     a = (I8 *)TO_VOIDP((uptr)p + UNFIX(n));
+    S_thread_start_code_write(tc, 0, 0, TO_VOIDP(a));
     *a = (I8)UNFIX(x);
-    S_thread_end_code_write();
+    S_thread_end_code_write(tc, 0, 0, TO_VOIDP(a));
 
     return Svoid;
 }
 
 static ptr s_set_code_word(p, n, x) ptr p, n, x; {
     I16 *a;
+    ptr tc = get_thread_context();
 
-    S_thread_start_code_write();
     a = (I16 *)TO_VOIDP((uptr)p + UNFIX(n));
+    S_thread_start_code_write(tc, 0, 0, TO_VOIDP(a));
     *a = (I16)UNFIX(x);
-    S_thread_end_code_write();
+    S_thread_end_code_write(tc, 0, 0, TO_VOIDP(a));
 
     return Svoid;
 }
 
 static ptr s_set_code_long(p, n, x) ptr p, n, x; {
     I32 *a;
+    ptr tc = get_thread_context();
 
-    S_thread_start_code_write();
     a = (I32 *)TO_VOIDP((uptr)p + UNFIX(n));
+    S_thread_start_code_write(tc, 0, 0, TO_VOIDP(a));
     *a = (I32)(Sfixnump(x) ? UNFIX(x) : Sinteger_value(x));
-    S_thread_end_code_write();
+    S_thread_end_code_write(tc, 0, 0, TO_VOIDP(a));
 
     return Svoid;
 }
 
 static void s_set_code_long2(p, n, h, l) ptr p, n, h, l; {
     I32 *a;
+    ptr tc = get_thread_context();
 
-    S_thread_start_code_write();
     a = (I32 *)TO_VOIDP((uptr)p + UNFIX(n));
+    S_thread_start_code_write(tc, 0, 0, TO_VOIDP(a));
     *a = (I32)((UNFIX(h) << 16) + UNFIX(l));
-    S_thread_end_code_write();
+    S_thread_end_code_write(tc, 0, 0, TO_VOIDP(a));
 }
 
 static ptr s_set_code_quad(p, n, x) ptr p, n, x; {
     I64 *a;
+    ptr tc = get_thread_context();
 
-    S_thread_start_code_write();
     a = (I64 *)TO_VOIDP((uptr)p + UNFIX(n));
+    S_thread_start_code_write(tc, 0, 0, TO_VOIDP(a));
     *a = Sfixnump(x) ? UNFIX(x) : S_int64_value("\\#set-code-quad!", x);
-    S_thread_end_code_write();
+    S_thread_end_code_write(tc, 0, 0, TO_VOIDP(a));
 
     return Svoid;
 }
 
 static ptr s_set_reloc(p, n, e) ptr p, n, e; {
     iptr *a;
+    ptr tc = get_thread_context();
 
-    S_thread_start_code_write();
+    S_thread_start_code_write(tc, 0, 0, TO_VOIDP(&CODERELOC(p)));
     a = (iptr *)(&RELOCIT(CODERELOC(p), UNFIX(n)));
     *a = Sfixnump(e) ? UNFIX(e) : Sinteger_value(e);
-    S_thread_end_code_write();
+    S_thread_end_code_write(tc, 0, 0, TO_VOIDP(&CODERELOC(p)));
 
     return e;
 }
@@ -947,10 +952,11 @@ static ptr s_flush_instruction_cache() {
 static ptr s_make_code(flags, free, name, arity_mark, n, info, pinfos)
                        iptr flags, free, n; ptr name, arity_mark, info, pinfos; {
     ptr co;
+    ptr tc = get_thread_context();
 
-    S_thread_start_code_write();
+    S_thread_start_code_write(tc, 0, 0, NULL);
 
-    co = S_code(get_thread_context(), type_code | (flags << code_flags_offset), n);
+    co = S_code(tc, type_code | (flags << code_flags_offset), n);
     CODEFREE(co) = free;
     CODENAME(co) = name;
     CODEARITYMASK(co) = arity_mark;
@@ -960,16 +966,18 @@ static ptr s_make_code(flags, free, name, arity_mark, n, info, pinfos)
       S_G.profile_counters = Scons(S_weak_cons(co, pinfos), S_G.profile_counters);
     }
 
-    S_thread_end_code_write();
+    S_thread_end_code_write(tc, 0, 0, NULL);
 
     return co;
 }
 
 static ptr s_make_reloc_table(codeobj, n) ptr codeobj, n; {
-    S_thread_start_code_write();
+    ptr tc = get_thread_context();
+
+    S_thread_start_code_write(tc, 0, 0, TO_VOIDP(&CODERELOC(codeobj)));
     CODERELOC(codeobj) = S_relocation_table(UNFIX(n));
     RELOCCODE(CODERELOC(codeobj)) = codeobj;
-    S_thread_end_code_write();
+    S_thread_end_code_write(tc, 0, 0, TO_VOIDP(&CODERELOC(codeobj)));
     return Svoid;
 }
 
@@ -1330,8 +1338,8 @@ static ptr s_set_collect_trip_bytes(n) ptr n; {
     return Svoid;
 }
 
-static void c_exit(UNUSED I32 status) {
-    S_abnormal_exit();
+static void c_exit(I32 status) {
+    exit(status);
 }
 
 #if defined(__STDC__) || defined(USE_ANSI_PROTOTYPES)
@@ -1756,6 +1764,7 @@ void S_prim5_init() {
     Sforeign_symbol("(cs)s_big_positive_bit_field", (void *)S_big_positive_bit_field);
     Sforeign_symbol("(cs)s_big_eq", (void *)S_big_eq);
     Sforeign_symbol("(cs)s_big_lt", (void *)S_big_lt);
+    Sforeign_symbol("(cs)s_big_trailing_zero_bits", (void *)S_big_trailing_zero_bits);
     Sforeign_symbol("(cs)s_bigoddp", (void *)s_bigoddp);
     Sforeign_symbol("(cs)s_div", (void *)S_div);
     Sforeign_symbol("(cs)s_float", (void *)s_float);
@@ -1773,6 +1782,7 @@ void S_prim5_init() {
     Sforeign_symbol("(cs)s_set_random_seed", (void *)s_set_random_seed);
     Sforeign_symbol("(cs)ss_trunc", (void *)S_trunc);
     Sforeign_symbol("(cs)ss_trunc_rem", (void *)s_trunc_rem);
+    Sforeign_symbol("(cs)s_rational", (void *)S_rational);
     Sforeign_symbol("(cs)sub", (void *)S_sub);
     Sforeign_symbol("(cs)rem", (void *)S_rem);
 #ifdef GETWD
@@ -2079,7 +2089,12 @@ static void s_free(uptr addr) {
 }
 
 #ifdef FEATURE_ICONV
-#ifdef WIN32
+#ifdef DISABLE_ICONV
+# define iconv_t int
+#define ICONV_OPEN(to, from) -1
+#define ICONV(cd, in, inb, out, outb) -1
+#define ICONV_CLOSE(cd) -1
+#elif defined(WIN32)
 typedef void *iconv_t;
 typedef iconv_t (*iconv_open_ft)(const char *tocode, const char *fromcode);
 typedef size_t (*iconv_ft)(iconv_t cd, char **inbuf, size_t *inbytesleft, char **outbuf, size_t *outbytesleft);
