@@ -360,7 +360,7 @@ void rktio_close_fds_after_fork(int len, int skip1, int skip2, int skip3);
 
 int rktio_system_fd_is_terminal(rktio_t *rktio, intptr_t fd);
 
-#ifdef RKTIO_USE_PTHREADS
+#if defined(RKTIO_USE_PTHREADS) && !defined(NO_PTHREAD_CANCEL)
 # define RKTIO_USE_PENDING_OPEN
 #endif
 
@@ -416,9 +416,11 @@ char *rktio_strndup(char *s, intptr_t len);
 
 #ifdef RKTIO_SYSTEM_UNIX
 void rktio_set_signal_handler(int sig_id, void (*proc)(int));
+void rktio_restore_modified_signal_handlers();
 #endif
 void rktio_forget_os_signal_handler(rktio_t *rktio);
 
+
 #ifdef RKTIO_SYSTEM_WINDOWS
 int rktio_system_time_is_dst(SYSTEMTIME *st, TIME_ZONE_INFORMATION *_tz);
 #endif
